/* FichothequeLib_Tools - Copyright (c) 2009-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.externalsource.ExternalSourceDef;
import net.fichotheque.extraction.DataKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.AddendaExtractFilterUnit;
import net.fichotheque.extraction.filterunit.AlbumExtractFilterUnit;
import net.fichotheque.extraction.filterunit.CorpusExtractFilterUnit;
import net.fichotheque.extraction.filterunit.DefaultIncludeUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.ThesaurusExtractFilterUnit;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.utils.FilterUnitUtils;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class FilterUnitListBuilder {

    private final List<FilterUnit> list = new ArrayList<FilterUnit>();

    public FilterUnitListBuilder() {
    }

    public FilterUnitListBuilder add(FilterUnit filterUnit) {
        list.add(filterUnit);
        return this;
    }

    public FilterUnitListBuilder addFieldKeyFilter(FieldKey fieldKey, Attributes attributes) {
        list.add(FilterUnitUtils.toFieldKeyFilterUnit(fieldKey, attributes));
        return this;
    }

    public FilterUnitListBuilder addFieldNameFilter(short category, String fieldName, Attributes attributes) {
        if (fieldName.endsWith("*")) {
            int length = fieldName.length();
            if (length == 1) {
                return this;
            }
            String prefix = fieldName.substring(0, length - 1);
            list.add(FilterUnitUtils.toFieldNamePrefixFilterUnit(category, prefix, attributes));
        } else {
            try {
                FieldKey fieldKey = FieldKey.parse(category, fieldName);
                addFieldKeyFilter(fieldKey, attributes);
            } catch (java.text.ParseException pe) {
            }
        }
        return this;
    }

    public FilterUnitListBuilder addLotFilter(TagNameInfo tagNameInfo, List<FilterUnit> filterUnitList) {
        list.add(FilterUnitUtils.toLotFilterUnit(tagNameInfo, filterUnitList));
        return this;
    }

    public FilterUnitListBuilder addEnteteFilter() {
        list.add(FilterUnitUtils.ENTETE_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addCorpsdeficheFilter() {
        list.add(FilterUnitUtils.CORPSDEFICHE_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addChronoFilter() {
        list.add(FilterUnitUtils.CHRONO_FILTERUNIT);
        return this;
    }

    public FilterUnitListBuilder addPhraseFilter(String phraseName) {
        list.add(FilterUnitUtils.toPhraseFilterUnit(phraseName));
        return this;
    }

    public FilterUnitListBuilder addDefaultFilter(ExtendedIncludeKey includeKey, Fichotheque fichotheque, Attributes attributes) {
        switch (includeKey.getCategory()) {
            case SubsetKey.CATEGORY_CORPUS:
                CorpusExtractDef corpusExtractDef = ExtractionBuilderUtils.toCorpusExtractDef(includeKey);
                if (attributes.isEmpty()) {
                    list.add(new DefaultCorpusExtractFilterUnit(corpusExtractDef, includeKey));
                } else {
                    addCorpusExtractFilter(corpusExtractDef, attributes);
                }
                break;
            case SubsetKey.CATEGORY_THESAURUS:
                ThesaurusExtractDef thesaurusExtractDef = ExtractionBuilderUtils.toThesaurusExtractDef(includeKey, fichotheque);
                if (attributes.isEmpty()) {
                    list.add(new DefaultThesaurusExtractFilterUnit(thesaurusExtractDef, includeKey));
                } else {
                    addThesaurusExtractFilter(thesaurusExtractDef, attributes);
                }
                break;
            case SubsetKey.CATEGORY_ALBUM:
                AlbumExtractDef albumExtractDef = ExtractionBuilderUtils.toAlbumExtractDef(includeKey);
                if (attributes.isEmpty()) {
                    list.add(new DefaultAlbumExtractFilterUnit(albumExtractDef, includeKey));
                } else {
                    addAlbumExtractFilter(albumExtractDef, attributes);
                }
                break;
            case SubsetKey.CATEGORY_ADDENDA:
                AddendaExtractDef addendaExtractDef = ExtractionBuilderUtils.toAddendaExtractDef(includeKey);
                if (attributes.isEmpty()) {
                    list.add(new DefaultAddendaExtractFilterUnit(addendaExtractDef, includeKey));
                } else {
                    addAddendaExtractFilter(addendaExtractDef, attributes);
                }
                break;
        }
        return this;
    }

    public FilterUnitListBuilder addCorpusExtractFilter(CorpusExtractDef corpusExtractDef, Attributes attributes) {
        list.add(FilterUnitUtils.toCorpusExtractFilterUnit(corpusExtractDef, attributes));
        return this;
    }

    public FilterUnitListBuilder addThesaurusExtractFilter(ThesaurusExtractDef thesaurusExtractDef, Attributes attributes) {
        list.add(FilterUnitUtils.toThesaurusExtractFilterUnit(thesaurusExtractDef, attributes));
        return this;
    }

    public FilterUnitListBuilder addAlbumExtractFilter(AlbumExtractDef albumExtractDef, Attributes attributes) {
        list.add(FilterUnitUtils.toAlbumExtractFilterUnit(albumExtractDef, attributes));
        return this;
    }

    public FilterUnitListBuilder addAddendaExtractFilter(AddendaExtractDef addendaExtractDef, Attributes attributes) {
        list.add(FilterUnitUtils.toAddendaExtractFilterUnit(addendaExtractDef, attributes));
        return this;
    }

    public FilterUnitListBuilder addFicheParentageFilter(FicheFilter ficheFilter, Attributes attributes, Collection<SubsetKey> subsetKeys) {
        list.add(FilterUnitUtils.toFicheParentageFilterUnit(ficheFilter, attributes, subsetKeys));
        return this;
    }

    public FilterUnitListBuilder addMasterMotcleFilter(MotcleFilter masterMotcleFilter, Attributes attributes) {
        list.add(FilterUnitUtils.toMasterMotcleFilterUnit(masterMotcleFilter, attributes));
        return this;
    }

    public FilterUnitListBuilder addDataFilter(DataKey dataKey, ExternalSourceDef externalSourceDef, Attributes attributes) {
        list.add(FilterUnitUtils.toDataFilterUnit(dataKey, externalSourceDef, attributes));
        return this;
    }

    public List<FilterUnit> toFilterUnitList() {
        return FilterUnitUtils.toImmutableList(list);
    }

    public final static FilterUnitListBuilder init() {
        return new FilterUnitListBuilder();
    }


    private static class DefaultCorpusExtractFilterUnit implements CorpusExtractFilterUnit, DefaultIncludeUnit {

        private final CorpusExtractDef corpusExtractDef;
        private final ExtendedIncludeKey includeKey;

        private DefaultCorpusExtractFilterUnit(CorpusExtractDef corpusExtractDef, ExtendedIncludeKey includeKey) {
            this.corpusExtractDef = corpusExtractDef;
            this.includeKey = includeKey;
        }

        @Override
        public CorpusExtractDef getCorpusExtractDef() {
            return corpusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return includeKey;
        }

    }


    private static class DefaultThesaurusExtractFilterUnit implements ThesaurusExtractFilterUnit, DefaultIncludeUnit {

        private final ThesaurusExtractDef thesaurusExtractDef;
        private final ExtendedIncludeKey includeKey;

        private DefaultThesaurusExtractFilterUnit(ThesaurusExtractDef thesaurusExtractDef, ExtendedIncludeKey includeKey) {
            this.thesaurusExtractDef = thesaurusExtractDef;
            this.includeKey = includeKey;
        }

        @Override
        public ThesaurusExtractDef getThesaurusExtractDef() {
            return thesaurusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return includeKey;
        }

    }


    private static class DefaultAlbumExtractFilterUnit implements AlbumExtractFilterUnit, DefaultIncludeUnit {

        private final AlbumExtractDef albumExtractDef;
        private final ExtendedIncludeKey includeKey;

        private DefaultAlbumExtractFilterUnit(AlbumExtractDef albumExtractDef, ExtendedIncludeKey includeKey) {
            this.albumExtractDef = albumExtractDef;
            this.includeKey = includeKey;
        }

        @Override
        public AlbumExtractDef getAlbumExtractDef() {
            return albumExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return includeKey;
        }

    }


    private static class DefaultAddendaExtractFilterUnit implements AddendaExtractFilterUnit, DefaultIncludeUnit {

        private final AddendaExtractDef addendaExtractDef;
        private final ExtendedIncludeKey includeKey;

        private DefaultAddendaExtractFilterUnit(AddendaExtractDef addendaExtractDef, ExtendedIncludeKey includeKey) {
            this.addendaExtractDef = addendaExtractDef;
            this.includeKey = includeKey;
        }

        @Override
        public AddendaExtractDef getAddendaExtractDef() {
            return addendaExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

        @Override
        public ExtendedIncludeKey getExtendedIncludeKey() {
            return includeKey;
        }

    }

}
